@@ -6,33 +6,33 @@ from typing import Dict
 from typing import List
 from typing import Union
 
+import requests
+
 from cachecontrol import CacheControl
 from cachecontrol.caches.file_cache import FileCache
 from cachecontrol.controller import logger as cache_control_logger
 from cachy import CacheManager
 from html5lib.html5parser import parse
-from requests import get
-from requests import session
-from requests.exceptions import TooManyRedirects
 
-from poetry.locations import CACHE_DIR
-from poetry.packages import Package
-from poetry.packages import dependency_from_pep_508
-from poetry.packages.utils.link import Link
-from poetry.semver import VersionConstraint
-from poetry.semver import VersionRange
-from poetry.semver import parse_constraint
-from poetry.semver.exceptions import ParseVersionError
+from poetry.core.packages import Dependency
+from poetry.core.packages import Package
+from poetry.core.packages import dependency_from_pep_508
+from poetry.core.packages.utils.link import Link
+from poetry.core.semver import VersionConstraint
+from poetry.core.semver import VersionRange
+from poetry.core.semver import parse_constraint
+from poetry.core.semver.exceptions import ParseVersionError
+from poetry.core.version.markers import parse_marker
+from poetry.locations import REPOSITORY_CACHE_DIR
 from poetry.utils._compat import Path
 from poetry.utils._compat import to_str
+from poetry.utils.helpers import download_file
 from poetry.utils.helpers import temporary_directory
-from poetry.utils.inspector import Inspector
 from poetry.utils.patterns import wheel_file_re
-from poetry.version.markers import InvalidMarker
-from poetry.version.markers import parse_marker
 
+from ..inspection.info import PackageInfo
 from .exceptions import PackageNotFound
-from .repository import Repository
+from .remote_repository import RemoteRepository
 
 
 try:
@@ -46,16 +46,18 @@ cache_control_logger.setLevel(logging.ERROR)
 logger = logging.getLogger(__name__)
 
 
-class PyPiRepository(Repository):
+class PyPiRepository(RemoteRepository):
 
     CACHE_VERSION = parse_constraint("1.0.0")
 
     def __init__(self, url="https://pypi.org/", disable_cache=False, fallback=True):
-        self._url = url
+        super(PyPiRepository, self).__init__(url.rstrip("/") + "/simple/")
+
+        self._base_url = url
         self._disable_cache = disable_cache
         self._fallback = fallback
 
-        release_cache_dir = Path(CACHE_DIR) / "cache" / "repositories" / "pypi"
+        release_cache_dir = REPOSITORY_CACHE_DIR / "pypi"
         self._cache = CacheManager(
             {
                 "default": "releases",
@@ -68,37 +70,24 @@ class PyPiRepository(Repository):
         )
 
         self._cache_control_cache = FileCache(str(release_cache_dir / "_http"))
-        self._session = CacheControl(session(), cache=self._cache_control_cache)
-        self._inspector = Inspector()
-
-        super(PyPiRepository, self).__init__()
-
         self._name = "PyPI"
 
     @property
-    def url(self):  # type: () -> str
-        return self._url
+    def session(self):
+        return CacheControl(requests.session(), cache=self._cache_control_cache)
 
-    @property
-    def authenticated_url(self):  # type: () -> str
-        return self._url
-
-    def find_packages(
-        self,
-        name,  # type: str
-        constraint=None,  # type: Union[VersionConstraint, str, None]
-        extras=None,  # type: Union[list, None]
-        allow_prereleases=False,  # type: bool
-    ):  # type: (...) -> List[Package]
+    def find_packages(self, dependency):  # type: (Dependency) -> List[Package]
         """
         Find packages on the remote server.
         """
+        constraint = dependency.constraint
         if constraint is None:
             constraint = "*"
 
         if not isinstance(constraint, VersionConstraint):
             constraint = parse_constraint(constraint)
 
+        allow_prereleases = dependency.allows_prereleases()
         if isinstance(constraint, VersionRange):
             if (
                 constraint.max is not None
@@ -109,122 +98,71 @@ class PyPiRepository(Repository):
                 allow_prereleases = True
 
         try:
-            info = self.get_package_info(name)
+            info = self.get_package_info(dependency.name)
         except PackageNotFound:
             self._log(
-                "No packages found for {} {}".format(name, str(constraint)),
+                "No packages found for {} {}".format(dependency.name, str(constraint)),
                 level="debug",
             )
             return []
 
         packages = []
+        ignored_pre_release_packages = []
 
         for version, release in info["releases"].items():
             if not release:
                 # Bad release
                 self._log(
                     "No release information found for {}-{}, skipping".format(
-                        name, version
+                        dependency.name, version
                     ),
                     level="debug",
                 )
                 continue
 
             try:
-                package = Package(name, version)
+                package = Package(info["info"]["name"], version)
             except ParseVersionError:
                 self._log(
                     'Unable to parse version "{}" for the {} package, skipping'.format(
-                        version, name
+                        version, dependency.name
                     ),
                     level="debug",
                 )
                 continue
 
             if package.is_prerelease() and not allow_prereleases:
+                if constraint.is_any():
+                    # we need this when all versions of the package are pre-releases
+                    ignored_pre_release_packages.append(package)
                 continue
 
             if not constraint or (constraint and constraint.allows(package.version)):
-                if extras is not None:
-                    package.requires_extras = extras
-
                 packages.append(package)
 
         self._log(
-            "{} packages found for {} {}".format(len(packages), name, str(constraint)),
+            "{} packages found for {} {}".format(
+                len(packages), dependency.name, str(constraint)
+            ),
             level="debug",
         )
 
-        return packages
+        return packages or ignored_pre_release_packages
 
     def package(
         self,
         name,  # type: str
         version,  # type: str
         extras=None,  # type: (Union[list, None])
-    ):  # type: (...) -> Union[Package, None]
-        if extras is None:
-            extras = []
-
-        release_info = self.get_release_info(name, version)
-        package = Package(name, version, version)
-        requires_dist = release_info["requires_dist"] or []
-        for req in requires_dist:
-            try:
-                dependency = dependency_from_pep_508(req)
-            except InvalidMarker:
-                # Invalid marker
-                # We strip the markers hoping for the best
-                req = req.split(";")[0]
-
-                dependency = dependency_from_pep_508(req)
-            except ValueError:
-                # Likely unable to parse constraint so we skip it
-                self._log(
-                    "Invalid constraint ({}) found in {}-{} dependencies, "
-                    "skipping".format(req, package.name, package.version),
-                    level="debug",
-                )
-                continue
-
-            if dependency.in_extras:
-                for extra in dependency.in_extras:
-                    if extra not in package.extras:
-                        package.extras[extra] = []
-
-                    package.extras[extra].append(dependency)
-
-            if not dependency.is_optional():
-                package.requires.append(dependency)
-
-        # Adding description
-        package.description = release_info.get("summary", "")
-
-        if release_info["requires_python"]:
-            package.python_versions = release_info["requires_python"]
-
-        if release_info["platform"]:
-            package.platform = release_info["platform"]
-
-        # Adding hashes information
-        package.files = release_info["files"]
-
-        # Activate extra dependencies
-        for extra in extras:
-            if extra in package.extras:
-                for dep in package.extras[extra]:
-                    dep.activate()
-
-                package.requires += package.extras[extra]
-
-        return package
+    ):  # type: (...) -> Package
+        return self.get_release_info(name, version).to_package(name=name, extras=extras)
 
     def search(self, query):
         results = []
 
         search = {"q": query}
 
-        response = session().get(self._url + "search", params=search)
+        response = requests.session().get(self._base_url + "search", params=search)
         content = parse(response.content, namespaceHTMLElements=False)
         for result in content.findall(".//*[@class='package-snippet']"):
             name = result.find("h3/*[@class='package-snippet__name']").text
@@ -272,7 +210,7 @@ class PyPiRepository(Repository):
 
         return data
 
-    def get_release_info(self, name, version):  # type: (str, str) -> dict
+    def get_release_info(self, name, version):  # type: (str, str) -> PackageInfo
         """
         Return the release information given a package name and a version.
 
@@ -280,7 +218,7 @@ class PyPiRepository(Repository):
         or retrieved from the remote server.
         """
         if self._disable_cache:
-            return self._get_release_info(name, version)
+            return PackageInfo.load(self._get_release_info(name, version))
 
         cached = self._cache.remember_forever(
             "{}:{}".format(name, version), lambda: self._get_release_info(name, version)
@@ -297,7 +235,19 @@ class PyPiRepository(Repository):
 
             self._cache.forever("{}:{}".format(name, version), cached)
 
-        return cached
+        return PackageInfo.load(cached)
+
+    def find_links_for_package(self, package):
+        json_data = self._get("pypi/{}/{}/json".format(package.name, package.version))
+        if json_data is None:
+            return []
+
+        links = []
+        for url in json_data["urls"]:
+            h = "sha256={}".format(url["digests"]["sha256"])
+            links.append(Link(url["url"] + "#" + h))
+
+        return links
 
     def _get_release_info(self, name, version):  # type: (str, str) -> dict
         self._log("Getting info for {} ({}) from PyPI".format(name, version), "debug")
@@ -307,16 +257,17 @@ class PyPiRepository(Repository):
             raise PackageNotFound("Package [{}] not found.".format(name))
 
         info = json_data["info"]
-        data = {
-            "name": info["name"],
-            "version": info["version"],
-            "summary": info["summary"],
-            "platform": info["platform"],
-            "requires_dist": info["requires_dist"],
-            "requires_python": info["requires_python"],
-            "files": [],
-            "_cache_version": str(self.CACHE_VERSION),
-        }
+
+        data = PackageInfo(
+            name=info["name"],
+            version=info["version"],
+            summary=info["summary"],
+            platform=info["platform"],
+            requires_dist=info["requires_dist"],
+            requires_python=info["requires_python"],
+            files=info.get("files", []),
+            cache_version=str(self.CACHE_VERSION),
+        )
 
         try:
             version_info = json_data["releases"][version]
@@ -324,14 +275,14 @@ class PyPiRepository(Repository):
             version_info = []
 
         for file_info in version_info:
-            data["files"].append(
+            data.files.append(
                 {
                     "file": file_info["filename"],
                     "hash": "sha256:" + file_info["digests"]["sha256"],
                 }
             )
 
-        if self._fallback and data["requires_dist"] is None:
+        if self._fallback and data.requires_dist is None:
             self._log("No dependencies found, downloading archives", level="debug")
             # No dependencies set (along with other information)
             # This might be due to actually no dependencies
@@ -348,25 +299,25 @@ class PyPiRepository(Repository):
                 urls[dist_type].append(url["url"])
 
             if not urls:
-                return data
+                return data.asdict()
 
             info = self._get_info_from_urls(urls)
 
-            data["requires_dist"] = info["requires_dist"]
+            data.requires_dist = info.requires_dist
 
-            if not data["requires_python"]:
-                data["requires_python"] = info["requires_python"]
+            if not data.requires_python:
+                data.requires_python = info.requires_python
 
-        return data
+        return data.asdict()
 
     def _get(self, endpoint):  # type: (str) -> Union[dict, None]
         try:
-            json_response = self._session.get(self._url + endpoint)
-        except TooManyRedirects:
+            json_response = self.session.get(self._base_url + endpoint)
+        except requests.exceptions.TooManyRedirects:
             # Cache control redirect loop.
             # We try to remove the cache and try again
-            self._cache_control_cache.delete(self._url + endpoint)
-            json_response = self._session.get(self._url + endpoint)
+            self._cache_control_cache.delete(self._base_url + endpoint)
+            json_response = self.session.get(self._base_url + endpoint)
 
         if json_response.status_code == 404:
             return None
@@ -375,9 +326,7 @@ class PyPiRepository(Repository):
 
         return json_data
 
-    def _get_info_from_urls(
-        self, urls
-    ):  # type: (Dict[str, List[str]]) -> Dict[str, Union[str, List, None]]
+    def _get_info_from_urls(self, urls):  # type: (Dict[str, List[str]]) -> PackageInfo
         # Checking wheels first as they are more likely to hold
         # the necessary information
         if "bdist_wheel" in urls:
@@ -412,24 +361,24 @@ class PyPiRepository(Repository):
             if universal_wheel is not None:
                 return self._get_info_from_wheel(universal_wheel)
 
-            info = {}
+            info = None
             if universal_python2_wheel and universal_python3_wheel:
                 info = self._get_info_from_wheel(universal_python2_wheel)
 
                 py3_info = self._get_info_from_wheel(universal_python3_wheel)
-                if py3_info["requires_dist"]:
-                    if not info["requires_dist"]:
-                        info["requires_dist"] = py3_info["requires_dist"]
+                if py3_info.requires_dist:
+                    if not info.requires_dist:
+                        info.requires_dist = py3_info.requires_dist
 
                         return info
 
                     py2_requires_dist = set(
                         dependency_from_pep_508(r).to_pep_508()
-                        for r in info["requires_dist"]
+                        for r in info.requires_dist
                     )
                     py3_requires_dist = set(
                         dependency_from_pep_508(r).to_pep_508()
-                        for r in py3_info["requires_dist"]
+                        for r in py3_info.requires_dist
                     )
                     base_requires_dist = py2_requires_dist & py3_requires_dist
                     py2_only_requires_dist = py2_requires_dist - py3_requires_dist
@@ -451,7 +400,7 @@ class PyPiRepository(Repository):
                         )
                         requires_dist.append(dep.to_pep_508())
 
-                    info["requires_dist"] = sorted(list(set(requires_dist)))
+                    info.requires_dist = sorted(list(set(requires_dist)))
 
             if info:
                 return info
@@ -469,9 +418,7 @@ class PyPiRepository(Repository):
 
         return self._get_info_from_sdist(urls["sdist"][0])
 
-    def _get_info_from_wheel(
-        self, url
-    ):  # type: (str) -> Dict[str, Union[str, List, None]]
+    def _get_info_from_wheel(self, url):  # type: (str) -> PackageInfo
         self._log(
             "Downloading wheel: {}".format(urlparse.urlparse(url).path.rsplit("/")[-1]),
             level="debug",
@@ -483,11 +430,9 @@ class PyPiRepository(Repository):
             filepath = Path(temp_dir) / filename
             self._download(url, str(filepath))
 
-            return self._inspector.inspect_wheel(filepath)
+            return PackageInfo.from_wheel(filepath)
 
-    def _get_info_from_sdist(
-        self, url
-    ):  # type: (str) -> Dict[str, Union[str, List, None]]
+    def _get_info_from_sdist(self, url):  # type: (str) -> PackageInfo
         self._log(
             "Downloading sdist: {}".format(urlparse.urlparse(url).path.rsplit("/")[-1]),
             level="debug",
@@ -499,16 +444,10 @@ class PyPiRepository(Repository):
             filepath = Path(temp_dir) / filename
             self._download(url, str(filepath))
 
-            return self._inspector.inspect_sdist(filepath)
+            return PackageInfo.from_sdist(filepath)
 
     def _download(self, url, dest):  # type: (str, str) -> None
-        r = get(url, stream=True)
-        r.raise_for_status()
-
-        with open(dest, "wb") as f:
-            for chunk in r.iter_content(chunk_size=1024):
-                if chunk:
-                    f.write(chunk)
+        return download_file(url, dest, session=self.session)
 
     def _log(self, msg, level="info"):
-        getattr(logger, level)("<comment>{}:</comment> {}".format(self._name, msg))
+        getattr(logger, level)("<debug>{}:</debug> {}".format(self._name, msg))
